<?php
/**
 * Created by PhpStorm.
 * User: Administrator
 * Date: 2019/6/14
 * Time: 16:23
 */

namespace App\Http\Controllers\V1;

use App\Repositories\PostRepositories;
use App\Traits\UserTrait;
use App\Transformers\Post\CommentTransformer;
use App\Transformers\Post\ListTransformer;
use App\Transformers\Post\ReplyTransformer;
use App\Transformers\Post\SuggestTransformer;
use Illuminate\Http\Request;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use League\Fractal\Manager;
use League\Fractal\Pagination\IlluminatePaginatorAdapter;
use League\Fractal\Resource\Collection;
use League\Fractal\Resource\Item;

class PostController extends Controller
{
    use UserTrait;
    public function __construct(PostRepositories $postRepositories)
    {
        $this->postRepositories = $postRepositories;
    }

    /**
     * 内容列表
     */
    public function index(Request $request)
    {
        $userInfo = $this->getUserInfo();
        if(empty($userInfo)){
            Log::info('获取用户信息失败');
            return jsonError('获取用户信息失败');
        }
        $list = $this->postRepositories->lists($request->all());
        $fractal = new Manager();
        $resource = new Collection($list, new ListTransformer($userInfo['uid']));
        $resource->setPaginator(new IlluminatePaginatorAdapter($list));
        $data = $fractal->createData($resource)->toArray();

        return jsonSuccess($data);
    }

    /**
     * 推荐内容列表
     */
    public function suggestPost(Request $request)
    {
        $userInfo = $this->getUserInfo();
        if(empty($userInfo)){
            Log::info('获取用户信息失败');
            return jsonError('获取用户信息失败');
        }
        $param = $request->all();
        $list = $this->postRepositories->suggestPost($param);
        $fractal = new Manager();
        $resource = new Collection($list, new SuggestTransformer($userInfo['uid']));
        $resource->setPaginator(new IlluminatePaginatorAdapter($list));
        $data = $fractal->createData($resource)->toArray();

        if(!(isset($param['current_page']) && $param['current_page'] > 1)){
            $newData = [];
            foreach($data['data'] as $key => $val){
                if($key == 1){
                    $newData[] = ['show_type' => 1];
                }
                $newData[] = $val;
            }
            $data['data'] = $newData;
        }
        return jsonSuccess($data);
    }

    /**
     * 评论列表
     */
    public function commentList(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'post_id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return jsonError($validator->errors()->first());
        }
        $list = $this->postRepositories->commentList($request->all());
        $fractal = new Manager();
        $resource = new Collection($list, new CommentTransformer());
        $resource->setPaginator(new IlluminatePaginatorAdapter($list));
        $data = $fractal->createData($resource)->toArray();

        return jsonSuccess($data);
    }

    /**
     * 回复列表
     */
    public function replyList(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required|integer',
        ]);
        if ($validator->fails()) {
            return jsonError($validator->errors()->first());
        }
        $detail = $this->postRepositories->commentDetail($request->all());
        if(!$detail){
            return jsonError('获取评论信息失败');
        }
        $list = $this->postRepositories->replyList($request->all());
        $fractal = new Manager();
        $resource = new Collection($list, new ReplyTransformer());
        $resource->setPaginator(new IlluminatePaginatorAdapter($list));
        $data = $fractal->createData($resource)->toArray();

        $data['data']['extra'] = [
            'id' => $detail['id'],
            'uid' => $detail['uid'],
            'username' => $detail['username'],
            'reply_count' => $detail->reply->count(),
            'avatar' => $detail['avatar'],
            'content' => $detail['is_delete']?'该评论已被删除':$detail['content'],
            'created_at' => Carbon::parse($detail['created_at'])->diffForHumans(),
        ];
        return jsonSuccess($data);
    }
}